﻿using Microsoft.SharePoint.Client;
using Microsoft.SharePoint.Client.Publishing;
using Microsoft.SharePoint.Client.Publishing.Navigation;
using Microsoft.SharePoint.Client.Taxonomy;
using Microsoft.SharePoint.Client.WebParts;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;

namespace Mavention.SPSNL14.Pages_WebPartsWeb {
    public partial class Default : System.Web.UI.Page {
        ClientContext clientContext;

        protected void Page_PreInit(object sender, EventArgs e) {
            Uri redirectUrl;
            switch (SharePointContextProvider.CheckRedirectionStatus(Context, out redirectUrl)) {
                case RedirectionStatus.Ok:
                    return;
                case RedirectionStatus.ShouldRedirect:
                    Response.Redirect(redirectUrl.AbsoluteUri, endResponse: true);
                    break;
                case RedirectionStatus.CanNotRedirect:
                    Response.Write("An error occurred while processing your request.");
                    Response.End();
                    break;
            }
        }

        protected void Page_Load(object sender, EventArgs e) {
            var spContext = SharePointContextProvider.Current.GetSharePointContext(Context);

            clientContext = spContext.CreateUserClientContextForSPHost();
        }

        protected void PublishingPage_Click(object sender, EventArgs e) {
            RunDeploymentStep(DeployPublishingPage, PublishingPageStatus);
        }

        protected void WebPart_Click(object sender, EventArgs e) {
            RunDeploymentStep(DeployWebPart, WebPartStatus);
        }

        private void RunDeploymentStep(Action<ClientContext> deploymentStep, System.Web.UI.WebControls.Label statusControl) {
            try {
                deploymentStep(clientContext);
                statusControl.Text = "DONE";
            }
            catch (Exception ex) {
                statusControl.Text = "ERROR";
                statusControl.ToolTip = ex.Message;
            }
        }

        private void DeployPublishingPage(ClientContext clientContext) {
            Web web = clientContext.Web;
            clientContext.Load(clientContext.Site.RootWeb, w => w.ServerRelativeUrl);
            clientContext.ExecuteQuery();

            // Get Page Layout
            File pageFromDocLayout = clientContext.Site.RootWeb.GetFileByServerRelativeUrl(String.Format("{0}/_catalogs/masterpage/BlankWebPartPage.aspx", clientContext.Site.RootWeb.ServerRelativeUrl.TrimEnd('/')));
            Microsoft.SharePoint.Client.ListItem pageLayoutItem = pageFromDocLayout.ListItemAllFields;
            clientContext.Load(pageLayoutItem);
            clientContext.ExecuteQuery();

            // Create Publishing Page
            PublishingWeb publishingWeb = PublishingWeb.GetPublishingWeb(clientContext, web);
            PublishingPage page = publishingWeb.AddPublishingPage(new PublishingPageInformation {
                Name = "My-Page.aspx",
                PageLayoutListItem = pageLayoutItem
            });
            clientContext.ExecuteQuery();

            // Set Page Title and Publish Page
            Microsoft.SharePoint.Client.ListItem pageItem = page.ListItem;
            pageItem["Title"] = "My Page";
            pageItem.Update();
            pageItem.File.CheckIn(String.Empty, CheckinType.MajorCheckIn);
            clientContext.ExecuteQuery();

            // Add Friendly URL
            // Requires Write permissions on Taxonomy
            TaxonomySession taxonomySession = TaxonomySession.GetTaxonomySession(clientContext);
            NavigationTermSet navigationTermSet = TaxonomyNavigation.GetTermSetForWeb(clientContext, web, "GlobalNavigationTaxonomyProvider", true);
            clientContext.Load(navigationTermSet);
            clientContext.ExecuteQuery();
            NavigationTermSet editableNavigationTermSet = navigationTermSet.GetAsEditable(taxonomySession);
            clientContext.Load(editableNavigationTermSet);
            clientContext.ExecuteQuery();
            ClientResult<string> furl = page.AddFriendlyUrl("my-page", editableNavigationTermSet, true);
            clientContext.ExecuteQuery();

            // Set Friendly URL Title
            NavigationTerm friendlyUrlTerm = editableNavigationTermSet.FindTermForUrl(furl.Value);
            friendlyUrlTerm.Title.Value = "My Page";
            friendlyUrlTerm.GetTaxonomyTermStore().CommitAll();
            clientContext.ExecuteQuery();
        }

        private void DeployWebPart(ClientContext clientContext) {
            Web web = clientContext.Web;

            clientContext.Load(web, w => w.ServerRelativeUrl, w => w.AllProperties);
            clientContext.ExecuteQuery();
            string pagesListName = web.AllProperties["__pageslistname"] as string;

            File page = web.GetFileByServerRelativeUrl(String.Format("{0}/{1}/My-Page.aspx", web.ServerRelativeUrl.TrimEnd('/'), pagesListName));
            page.CheckOut();

            // Requires Full Control permissions on the Web
            LimitedWebPartManager wpmgr = page.GetLimitedWebPartManager(PersonalizationScope.Shared);
            WebPartDefinition wpd = wpmgr.ImportWebPart(Properties.Resources.CEWP);
            wpmgr.AddWebPart(wpd.WebPart, "Header", 0);

            page.CheckIn(String.Empty, CheckinType.MajorCheckIn);
            clientContext.ExecuteQuery();
        }
    }
}